@@ -6,36 +6,62 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
                            "OR", "RR"),
                     incr=0.5, allincr=FALSE, addincr=FALSE, allstudies=FALSE,
                     MH.exact=FALSE, RR.cochrane=FALSE,
-                    warn=TRUE){
+                    level=0.95, level.comb=level,
+                    warn=TRUE
+                    ){
   
   
   if (is.null(data)) data <- sys.frame(sys.parent())
   ##
+  ## Catch event.e, n.e, event.c, n.e, studlab (possibly) from data:
+  ##
   mf <- match.call()
   mf$data <- mf$subset <- mf$method <- mf$sm <- NULL
-  mf$incr <- mf$allincr <- mf$allstudies <- mf$MH.exact <- NULL
+  mf$incr <- mf$allincr <- mf$addincr <- mf$allstudies <- NULL
+  mf$MH.exact <- mf$RR.cochrane <- NULL
+  mf$level <- mf$level.comb <- mf$warn <- NULL
   mf[[1]] <- as.name("data.frame")
   mf <- eval(mf, data)
   ##
-  if (!is.null(subset)) mf <- mf[subset,]
+  ## Catch subset (possibly) from data:
   ##
-  event.e <- mf$event.e
-  n.e <- mf$n.e
-  event.c <- mf$event.c
-  n.c <- mf$n.c
+  mf2 <- match.call()
+  mf2$event.e <- mf2$n.e <- NULL
+  mf2$event.c <- mf2$n.c <- NULL
+  mf2$studlab <- NULL 
+  mf2$data <- mf2$method <- mf2$sm <- NULL
+  mf2$incr <- mf2$allincr <- mf2$addincr <- mf2$allstudies <- NULL
+  mf2$MH.exact <- mf2$RR.cochrane <- NULL
+  mf2$level <- mf2$level.comb <- mf2$warn <- NULL
+  mf2[[1]] <- as.name("data.frame")
   ##
-  if (!missing(studlab)) studlab <- as.character(mf$studlab)
-
+  mf2 <- eval(mf2, data)
+  ##
+  if (!is.null(mf2$subset))
+    if ((is.logical(mf2$subset) & (sum(mf2$subset) > length(mf$event.e))) ||
+        (length(mf2$subset) > length(mf$event.e)))
+      stop("Length of subset is larger than number of trials.")
+    else
+      mf <- mf[mf2$subset,]
+  ##
+  event.e <- as.numeric(mf$event.e)
+  n.e <- as.numeric(mf$n.e)
+  event.c <- as.numeric(mf$event.c)
+  n.c <- as.numeric(mf$n.c)
+  ##
+  if (!missing(studlab))
+    studlab <- as.character(mf$studlab)
+  else
+    studlab <- row.names(mf)
+  
   
   k.all <- length(event.e)
   ##
-  if (k.all == 0) stop("event.e = numeric(0)")
-  ##
-  if (missing(studlab)) studlab <- as.character(1:k.all)
+  if (k.all == 0) stop("No trials to combine in meta-analysis.")
 
 
-  if (match(sm, c("OR", "RD", "RR"), nomatch=0) == 0)
-    stop("possible summary measures are \"OR\", \"RD\" and \"RR\"")
+  if (match(sm, c("OR", "RD", "RR", "AS"), nomatch=0) == 0)
+    stop("possible summary measures are \"OR\", \"RD\", \"RR\", and \"AS\"")
   ##
   if (!(is.numeric(event.e) & is.numeric(n.e) &
         is.numeric(event.c) & is.numeric(n.c)))
@@ -51,8 +77,36 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   ##
   if (length(studlab) != k.all)
     stop("Number of studies and labels are different")
-
-
+  ##
+  if (!is.numeric(incr)){
+    ##
+    iincr <- charmatch(tolower(incr),
+                       c("tacc"), nomatch = NA)
+    ##
+    if(is.na(iincr))
+      stop("incr should be numeric or the character string \"TACC\"")
+    ##
+    incr <- c("TACC")[iincr]
+  }
+  
+  
+  ##
+  ## Check for levels of confidence interval
+  ##
+  if (!is.numeric(level) | length(level)!=1)
+    stop("parameter 'level' must be a numeric of length 1")
+  if (level <= 0 | level >= 1)
+    stop("parameter 'level': no valid level for confidence interval")
+  ##
+  if (!is.numeric(level.comb) | length(level.comb)!=1)
+    stop("parameter 'level.comb' must be a numeric of length 1")
+  if (level.comb <= 0 | level.comb >= 1)
+    stop("parameter 'level.comb': no valid level for confidence interval")
+  
+  
+  if (sm == "AS") method <- "Inverse"
+  
+  
   imeth <- charmatch(tolower(method),
                      c("inverse", "mh", "peto"), nomatch = NA)
   ##
@@ -62,18 +116,22 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   method <- c("Inverse", "MH", "Peto")[imeth]
   ##
   if (method == "Peto" & sm != "OR")
-    stop("Peto's method only possible with 'sm=OR'")
+    stop("Peto's method only possible with \"sm=OR\"")
   ##
   if (k.all == 1 & method == "MH"){
-    warning("For single trials, inverse variance method used instead of Mantel Haenszel method.")
+    warning("For a single trial, inverse variance method used instead of Mantel Haenszel method.")
     method <- "Inverse"
   }
-
+  
   
   ##
-  ## Include non-informative trials?
   ##
-  if (sm == "RD") incl <- rep(1, k.all)
+  ## Include non-informative trials?
+  ## (i.e. trials with either zero or all events in both groups)
+  ##
+  ##
+  if (sm == "RD" | sm == "AS")
+    incl <- rep(1, k.all)
   else{
     if (allstudies) incl <- rep(1, k.all)
     else
@@ -84,45 +142,111 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   ## k: effective number of trials
   ##
   k <- sum(!is.na(incl))
-
-
+  
+  
+  ##
   ##
   ## Sparse computation
   ##
+  ##
   sel <- switch(sm,
                 OR=((n.e - event.e) == 0 | event.e == 0 |
                     (n.c - event.c) == 0 | event.c == 0),
                 RD=((n.e - event.e) == 0 | event.e == 0 |
                     (n.c - event.c) == 0 | event.c == 0),
-                RR=(event.e == 0 | event.c == 0))
+                RR=(event.e == 0 | event.c == 0),
+                AS=rep(FALSE, length(event.e)))
   ##
   sel[is.na(incl)] <- FALSE
   ##
   sparse <- any(sel)
   ##
+  ## No need to add anything to cell counts for
+  ##  (i)  arcsine difference
+  ##  (ii) Peto method
+  ## as summary measure.
+  ## Accordingly, no warning will be printed.
+  ##
+  warn2 <- !(sm == "AS" | method == "Peto")
+  ##
   if (addincr){
-    i <- rep(incr, k.all)
-    if (warn)
-      warning(paste(incr, "added to each cell frequency of all studies"))
+    ##
+    if (is.numeric(incr)){
+      incr.e <- rep(incr, k.all)
+      incr.c <- rep(incr, k.all)
+      ##
+      if (warn & warn2)
+        warning(paste("Increment", incr, "added to each cell frequency of all studies"))
+    }
+    else{
+      if (incr=="TACC"){
+        ##
+        ## Treatment arm continuity correction:
+        ##
+        incr.e <- n.e/(n.e+n.c)
+        incr.c <- n.c/(n.e+n.c)
+        ##
+        if (warn & warn2)
+          warning("Treatment arm continuity correction applied to all studies")
+      }
+    }
+    ##
   }
   else{
     if (sparse){
       if (allincr){
-        i <- rep(incr, k.all)
-        if (warn)
-          warning(paste(incr, "added to each cell frequency of all studies"))
+        ##
+        if (is.numeric(incr)){
+          incr.e <- rep(incr, k.all)
+          incr.c <- rep(incr, k.all)
+          ##
+          if (warn & warn2)
+            warning(paste("Increment", incr, "added to each cell frequency of all studies"))
+        }
+        else{
+          if (incr=="TACC"){
+            ##
+            ## Treatment arm continuity correction:
+            ##
+            incr.e <- n.e/(n.e+n.c)
+            incr.c <- n.c/(n.e+n.c)
+            ##
+            if (warn & warn2)
+              warning("Treatment arm continuity correction applied to all studies")
+          }
+        }
       }
       else{
         ##
         ## Bradburn, Deeks, Altman, Stata-procedure "metan":
         ## & SAS PROC FREQ (for method="Inverse")
         ##
-        i <- incr*sel
-        if (warn)
-          warning(paste(incr, "added to each cell in 2x2 tables with zero cell frequencies"))
+       ##
+        if (is.numeric(incr)){
+          incr.e <- incr*sel
+          incr.c <- incr*sel
+          ##
+           if (warn & warn2)
+            warning(paste("Increment", incr, "added to each cell in 2x2 tables with zero cell frequencies"))
+        }
+        else{
+          if (incr=="TACC"){
+            ##
+            ## Treatment arm continuity correction:
+            ##
+            incr.e <- n.e/(n.e+n.c)*sel
+            incr.c <- n.c/(n.e+n.c)*sel
+            ##
+            if (warn & warn2)
+              warning("Treatment arm continuity correction applied to studies with zero cell frequencies")
+          }
+        }
       }
     }
-    else i <- rep(0, k.all)
+    else{
+      incr.e <- rep(0, k.all)
+      incr.c <- rep(0, k.all)
+    }
   }
   
   
@@ -136,19 +260,26 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   n22 <- n2. - n21
   n.1 <- n11 + n21
   n.2 <- n12 + n22
-
+  
   
   Q.CMH <- (sum(n11 - n1.*n.1/n.., na.rm=TRUE)^2/
             sum(n1.*n2.*n.1*n.2/n..^3, na.rm=TRUE))
-
-
+  
+  
+  ##
+  ##
+  ## Estimation of treatment effects in individual trials
+  ##
+  ##
   if (sm == "OR"){
     if (method == "MH" || method == "Inverse"){
       ## 
       ## Cooper & Hedges (1994), p. 251-2
       ## 
-      TE <- log(((n11+i)*(n22+i)) / ((n12+i)*(n21+i)))
-      varTE <- 1/(n11+i) + 1/(n12+i) + 1/(n21+i) + 1/(n22+i)
+      TE <- log(((n11+incr.e)*(n22+incr.c)) /
+                ((n12+incr.e)*(n21+incr.c)))
+      varTE <- (1/(n11+incr.e) + 1/(n12+incr.e) +
+                1/(n21+incr.c) + 1/(n22+incr.c))
     }
     else if (method == "Peto"){
       ## 
@@ -167,12 +298,16 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     ## Cooper & Hedges (1994), p. 247-8
     ## 
     if (!RR.cochrane){
-      TE <- log(((n11+i)/(n1.+i))/((n21+i)/(n2.+i)))
-      varTE <- 1/(n11+i) - 1/(n1.+i) + 1/(n21+i) - 1/(n2.+i)
+      TE <- log(((n11+incr.e)/(n1.+incr.e))/
+                ((n21+incr.c)/(n2.+incr.c)))
+      varTE <- (1/(n11+incr.e) - 1/(n1.+incr.e) +
+                1/(n21+incr.c) - 1/(n2.+incr.c))
       }
     else{
-      TE <- log(((n11+i)/(n1.+2*i))/((n21+i)/(n2.+2*i)))
-      varTE <- 1/(n11+i) - 1/(n1.+2*i) + 1/(n21+i) - 1/(n2.+2*i)
+      TE <- log(((n11+incr.e)/(n1.+2*incr.e))/
+                ((n21+incr.c)/(n2.+2*incr.c)))
+      varTE <- (1/(n11+incr.e) - 1/(n1.+2*incr.e) +
+                1/(n21+incr.c) - 1/(n2.+2*incr.c))
     }
   }
   else if (sm == "RD"){
@@ -180,22 +315,37 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     ## Cooper & Hedges (1994), p. 246-7
     ## 
     ##TE <- (n11+i)/(n1.+2*i) - (n21+i)/(n2.+2*i)
+    ##
     TE <- n11/n1. - n21/n2.
-    varTE <- (n11+i)*(n12+i)/(n1.+2*i)^3 +
-      (n21+i)*(n22+i)/(n2.+2*i)^3
+    varTE <- (n11+incr.e)*(n12+incr.e)/(n1.+2*incr.e)^3 +
+      (n21+incr.c)*(n22+incr.c)/(n2.+2*incr.c)^3
+  }
+  else if (sm == "AS"){
+    ## 
+    ## Gerta Ruecker, IMBI, 2005
+    ## 
+    TE <- asin(sqrt(n11/n1.)) - asin(sqrt(n21/n2.))
+    varTE <- 0.25*(1/n1. + 1/n2.)
   }
 
   
+  ##
   ##
   ## Calculate random effects estimate:
   ##
+  ##
   m <- metagen(TE, sqrt(varTE))
   ##
   TE.random <- m$TE.random
   seTE.random <- m$seTE.random
   w.random <- m$w.random
-
   
+  
+  ##
+  ##
+  ## Calculate fixed effect estimate:
+  ##
+  ##
   if (method == "Inverse" || method=="Peto"){
     w.fixed <- m$w.fixed
     TE.fixed <- m$TE.fixed
@@ -205,7 +355,8 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     if (!is.logical(MH.exact))
       stop("MH.exact must be of type 'logical'")
     ##
-    i <- i*(!MH.exact)
+    incr.e <- incr.e*(!MH.exact)
+    incr.c <- incr.c*(!MH.exact)
     ##
     if (sm == "OR"){
       ## 
@@ -214,10 +365,10 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
       ## Bradburn, Deeks, Altman, Stata-procedure "metan"
       ## und RevMan 3.1 (MH.exact==FALSE)
       ## 
-      A <- (n11+i)*(n22+i)/(n..+4*i)
-      B <- (n11+i + n22+i)/(n..+4*i)
-      C <- (n12+i)*(n21+i)/(n..+4*i)
-      D <- (n12+i + n21+i)/(n..+4*i)
+      A <- (n11+incr.e)*(n22+incr.c)/(n..+2*incr.e+2*incr.c)
+      B <- (n11+incr.e + n22+incr.c)/(n..+2*incr.e+2*incr.c)
+      C <- (n12+incr.e)*(n21+incr.c)/(n..+2*incr.e+2*incr.c)
+      D <- (n12+incr.e + n21+incr.c)/(n..+2*incr.e+2*incr.c)
       ##
       ## Cooper & Hedges (1994), p. 265-6
       ##
@@ -235,15 +386,17 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
       ##
       ## Bradburn, Deeks, Altman, Stata-procedure "metan"
       ## (MH.exact==FALSE)
-      ## 
-      D <- ((n1.+2*i)*(n2.+2*i)*(n.1+2*i) -
-            (n11+i)*(n21+i)*(n..+4*i))/(n..+4*i)^2
-      R <- (n11+i)*(n2.+2*i)/(n..+4*i)
-      S <- (n21+i)*(n1.+2*i)/(n..+4*i)
+      ##
+      D <- ((n1.+2*incr.e)*(n2.+2*incr.c)*(n.1+incr.e+incr.c) -
+            (n11+incr.e)*(n21+incr.c)*(n..+2*incr.e+2*incr.c))/
+              (n..+2*incr.e+2*incr.c)^2
+      R <- (n11+incr.e)*(n2.+2*incr.c)/(n..+2*incr.e+2*incr.c)
+      S <- (n21+incr.c)*(n1.+2*incr.e)/(n..+2*incr.e+2*incr.c)
       ##
       w.fixed <- S
       TE.fixed <- log(sum(R, na.rm=TRUE)/sum(S, na.rm=TRUE))
-      varTE.fixed <- sum(D, na.rm=TRUE)/(sum(R, na.rm=TRUE)*sum(S, na.rm=TRUE))
+      varTE.fixed <- sum(D, na.rm=TRUE)/(sum(R, na.rm=TRUE)*
+                              sum(S, na.rm=TRUE))
     }
     else if (sm == "RD"){
       ##
@@ -252,9 +405,10 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
       ## Bradburn, Deeks, Altman, Stata-procedure "metan"
       ## und RevMan 3.1 (MH.exact==FALSE)
       ## 
-      R <- ((n11+i)*(n12+i)*(n2.+2*i)^3 +
-            (n21+i)*(n22+i)*(n1.+2*i)^3)/
-              ((n1.+2*i)*(n2.+2*i)*(n..+4*i)^2)
+      R <- ((n11+incr.e)*(n12+incr.e)*(n2.+2*incr.c)^3 +
+            (n21+incr.c)*(n22+incr.c)*(n1.+2*incr.e)^3)/
+              ((n1.+2*incr.e)*(n2.+2*incr.c)*(n..+2*incr.e+2*incr.c)^2)
+      ##
       S <- n1.*n2./n..
       ##
       w.fixed <- S
@@ -262,8 +416,6 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
       varTE.fixed <- sum(R, na.rm=TRUE)/sum(S, na.rm=TRUE)^2
     }
   }
-
-
   ##
   ## Modify fixed effects estimate:
   ##
@@ -277,8 +429,11 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
 
   
   ##
+  ##
+  ## Calculate Cochrane Q (heterogeneity statistic)
   ## Cooper & Hedges (1994), p. 274-5
   ##
+  ##
   if (!is.na(TE.fixed)){
     Q <- sum(1/varTE*(TE-TE.fixed)^2, na.rm=TRUE)
     ##
@@ -292,7 +447,7 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     Q <- NA
     tau2 <- NA
   }
-
+  
   
   res <- list(event.e=event.e, n.e=n.e,
               event.c=event.c, n.c=n.c,
@@ -311,6 +466,10 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
               allstudies=allstudies,
               MH.exact=MH.exact,
               RR.cochrane=RR.cochrane,
+              incr.e=incr.e,
+              incr.c=incr.c,
+              level=level,
+              level.comb=level.comb,
               call=match.call(),
               warn=warn)
   
